/*
 * This program is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License, version 2.1 as published by the Free Software
 * Foundation.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, you can obtain a copy at http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * or from the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * Copyright (c) 2006 - 2009 Pentaho Corporation and Contributors.  All rights reserved.
 */

package org.pentaho.reporting.libraries.formula.function.logical;

import org.pentaho.reporting.libraries.formula.function.AbstractFunctionDescription;
import org.pentaho.reporting.libraries.formula.function.FunctionCategory;
import org.pentaho.reporting.libraries.formula.typing.Type;
import org.pentaho.reporting.libraries.formula.typing.coretypes.LogicalType;
import org.pentaho.reporting.libraries.formula.typing.coretypes.AnyType;

/**
 * Creation-Date: 04.11.2006, 18:28:55
 * 
 * @author Thomas Morgner
 */
public class IfFunctionDescription extends AbstractFunctionDescription
{
  private static final long serialVersionUID = 5553370162761578407L;

  public IfFunctionDescription()
  {
    super("IF", "org.pentaho.reporting.libraries.formula.function.logical.If-Function");
  }

  public int getParameterCount()
  {
    return 3;
  }

  public Type getParameterType(final int position)
  {
    if (position == 0)
    {
      return LogicalType.TYPE;
    }
    else
    {
      return AnyType.TYPE;
    }
  }

  public Type getValueType()
  {
    return AnyType.TYPE;
  }

  /**
   * Defines, whether the parameter at the given position is mandatory. A
   * mandatory parameter must be filled in, while optional parameters need not
   * to be filled in.
   * 
   * @return
   */
  public boolean isParameterMandatory(final int position)
  {
    if (position == 1 || position == 2)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public FunctionCategory getCategory()
  {
    return LogicalFunctionCategory.CATEGORY;
  }

}
